/* Copyright (c) 1997-2010
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: bounding_box.cc 9502 2010-02-24 00:26:29Z gawrilow $
*/

#include <polymake.h>
#include <Matrix.h>

namespace polymake { namespace common {

   template <typename _Matrix, typename Coord>
   Matrix<Coord> bounding_box(const GenericMatrix<_Matrix,Coord>& V)
   {
      const int d=V.cols();
      Matrix<Coord> BB(2,d);
      if (V.rows()) {
	 typename Entire< Rows<_Matrix> >::const_iterator r=entire(rows(V.top()));
	 BB[0]=*r;
	 BB[1]=*r;
	 while (!(++r).at_end()) {
	    typename _Matrix::row_type::const_iterator c=r->begin();
	    for (int j=0; j<d; ++j, ++c)
	       assign_min_max(BB(0,j), BB(1,j), *c);
	 }
      }
      return BB;
   }

   template <typename Coord>
   void extend_bounding_box(Matrix<Coord>& BB, const Matrix<Coord>& BB2)
   {
      if (BB.rows()) {
	 const int d=BB.cols();
	 const Coord *src=concat_rows(BB2).begin();
	 Coord *dst=concat_rows(BB).begin();
	 for (int j=0; j<d; ++j, ++dst, ++src) assign_min(*dst, *src);
	 for (int j=0; j<d; ++j, ++dst, ++src) assign_max(*dst, *src);
      } else {
	 BB=BB2;
      }
   }

   UserFunctionTemplate4perl("# @category Utilities"
                             "# Compute a column-wise bounding box for the give matrix"
                             "# @param Matrix m"
                             "# @return Matrix : row(0) contains lower bounds, row(1) contains upper bounds",
                             "bounding_box(Matrix)");

   FunctionTemplate4perl("extend_bounding_box(Matrix& Matrix) : void");
} }
