/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: check_int_limit.cc 9316 2009-09-15 20:38:33Z benmuell $
*/

#include <polymake.h>
#include <Integer.h>
#include <Matrix.h>
#include <Vector.h>

namespace polymake { namespace common {

   template <typename Vector>
   bool check_int_limit(const GenericVector<Vector,Integer>& V)
   {
      for (typename Entire<Vector>::const_iterator x=entire(V.top()); !x.at_end(); ++x)
    	 if ( *x > std::numeric_limits<int>::max() || *x < std::numeric_limits<int>::min() ) 
            return false;
      return true;
   }

   template <typename Matrix>
   bool check_int_limit(const GenericMatrix<Matrix,Integer>& M)
   {
      return check_int_limit(concat_rows(M));
   }

   FunctionTemplate4perl("check_int_limit(Vector<Integer>)");
   FunctionTemplate4perl("check_int_limit(Matrix<Integer>)");
} }
