/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: incidence_tools.cc 9298 2009-09-02 23:10:56Z gawrilow $
*/

#define INDEX_CHECKS 1

#include <IncidenceMatrix.h>
#include <Set.h>
#include <polymake.h>

namespace polymake { namespace common {

   template <typename IMatrix, typename ISet>
   Set<int> incident_rows(const GenericIncidenceMatrix<IMatrix>& IM, const ISet& S)
   {
      Set<int> result;
      accumulate_in(entire(select(cols(IM),S)),operations::add(),result);
      return result;
   }

   template <typename IMatrix, typename ISet>
   Set<int> not_incident_rows(const GenericIncidenceMatrix<IMatrix>& IM, const ISet& S)
   {
      Set<int> result(sequence(0,IM.rows()));
      accumulate_in(entire(select(cols(IM),S)),operations::sub(),result);
      return result;
   }

   template <typename IMatrix, typename ISet>
   Set<int> common_rows(const GenericIncidenceMatrix<IMatrix>& IM, const ISet& S)
   {
      Set<int> result;
      typename Entire<ISet>::const_iterator s=entire(S);
      if (!s.at_end()) {
	 result=IM.col(*s);
	 while (!(++s).at_end()) result *= IM.col(*s);
      }
      return result;
   }

   FunctionTemplate4perl("incident_rows(IncidenceMatrix, *)");
   FunctionTemplate4perl("not_incident_rows(IncidenceMatrix, *)");
   FunctionTemplate4perl("common_rows(IncidenceMatrix, *)");
} }
