/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: hypersimplex.cc 9527 2010-03-04 17:52:36Z herr $
*/

#include <hypersimplex.h>
#include <Rational.h>
#include <Matrix.h>
#include <PowerSet.h>

namespace polymake { namespace polytope {

perl::Object hypersimplex(int k, int d)
{
   if (d < 2)
      throw std::runtime_error("hypersimplex: dimension >= 2 required");
   if (k < 1 || k >= d)
      throw std::runtime_error("hypersimplex: 1 <= k < d required");

   perl::Object p("Polytope<Rational>");
   p.set_description() << "(" << k << "," << d << ")-hypersimplex" << endl;

   p.take("AMBIENT_DIM") << d;
   p.take("DIM") << d-1;

   // we already know the number of vertices
   const int n=Integer::binom(d,k);
   p.take("N_VERTICES") << n;

   Matrix<Rational> Vertices(n,d+1);
   Rows< Matrix<Rational> >::iterator v=rows(Vertices).begin();
   Subsets_of_k<sequence> enumerator(range(1,d), k);
   for (Subsets_of_k<sequence>::iterator s=entire(enumerator); !s.at_end(); ++s, ++v) {
      (*v)[0]=1;
      v->slice(*s).fill(1);
   }
   p.take("VERTICES") << Vertices;

   return p;
}

UserFunction4perl("# @category Producing from scratch"
		  "# Produce the hypersimplex &Delta;(k,d), that is the the convex hull of all 0/1-vector in R<sup>n</sup>"
		  "# with exactly //k// 1s."
		  "# Note that the output is never full-dimensional."
		  "# @param Int k number of 1s"
		  "# @param Int d ambient dimension"
		  "# @return Polytope",
		  &hypersimplex, "hypersimplex");
} }
