/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: revert.cc 9527 2010-03-04 17:52:36Z herr $
*/

#include <polymake.h>
#include <transform.h>

namespace polymake { namespace polytope {

   template <typename Scalar>
   perl::Object revert(perl::Object p_in)
   {

      const Matrix<Scalar> RT_v=p_in.get_attachment("REVERSE_TRANSFORMATION");

      perl::Object p_out=transform<Scalar>(p_in, RT_v, false);
      p_out.set_description() << "Polytope reversely transformed from " << p_in.name() << endl;

      return p_out;
   }

   UserFunctionTemplate4perl("# @category Transforming a polyhedron"
			     "# Apply a reverse transformation to a given polyhedron //P//."
			     "# All transformation clients keep track of the polytope's history."
			     "# They write or update the attachment REVERSE_TRANSFORMATION."
			     "# "
			     "# Applying revert to the transformed polytope reconstructs the original polyhedron."
			     "# @param Polytope P a (transformed) polytope"
			     "# @return Polytope",
			     "revert<Scalar> (Polytope<Scalar>)");
} }
