/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: 2D_tools.tcc 9298 2009-09-02 23:10:56Z gawrilow $
*/

namespace polymake { namespace topaz {

template <typename Complex, typename VertexSet>
bool is_ball_or_sphere(const Complex& C, const GenericSet<VertexSet>& V, int2type<2>)
{
   HasseDiagram HD=pure_hasse_diagram(C);

   // check whether C is a pseudo_manifold and compute the boundary B
   std::list< Set<int> > B;
   bool is_PM = is_pseudo_manifold(HD, true, back_inserter(B));
   if (!is_PM) return false;

   // check whether B is a sphere or empty
   const bool B_is_empty = B.empty();
   if (!B_is_empty && !is_ball_or_sphere(B,int2type<1>())) return false;

   // S:= C + B*v for a vertex v not contained in C
   // note: S is a sphere <=> C is ball or sphere
   // check euler char of S
   // if (B.empty())  S = -1 + #vert - #edges_of_C + #C
   // else            S = -1 + (#vert + 1) - (#edges_of_C + #B) + (#C + #B)
   int euler_char= V.top().size() - HD.nodes_of_dim(-2).size() + C.size();
   if (B_is_empty) --euler_char;

   if (euler_char!=1) return false;

   return true;
}

} }

// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
