/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: volume.cc 9420 2010-01-15 13:25:25Z sherrmann $
*/

#include <Rational.h>
#include <polymake.h>
#include <Array.h>
#include <Set.h>
#include <Matrix.h>
#include <Vector.h>
#include <linalg.h>

namespace polymake { namespace topaz {

Rational volume(perl::Object p)
{
  const int dim=p.give("DIM");
  const Array< Set<int> > F=p.give("FACETS");
  Matrix<Rational> GR=p.give("GEOMETRIC_REALIZATION");
  Vector<Rational> homogenization(GR.rows(),Rational(1));
  GR = homogenization | GR;
  
  Rational volume(0);
  for (Entire< Array< Set<int> > >::const_iterator f=entire(F); !f.at_end(); ++f) {
    Rational vol = abs(det( GR.minor(*f,All) ));
    if (vol == Rational(0))
      cerr << "volume: WARNING simplex of volume 0.\n";
    volume += vol;
  }
  volume /= Integer::fac(dim);
  
  return volume;
}
  

Function4perl(&volume,"volume");

} }
