/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: poly2trop.cc 9553 2010-03-09 18:16:07Z kulas $
*/

#include <polymake.h>
#include <list>
#include <Rational.h>
#include <Vector.h>
#include <Matrix.h>
#include <Array.h>

namespace polymake { namespace tropical {

perl::Object poly2trop(perl::Object p) {
  const bool bounded=p.give("BOUNDED");

  if (!bounded)
    throw std::runtime_error("input polyhedron not bounded");

  Matrix<Rational> V=p.give("VERTICES | POINTS");
  V.col(0)=zero_vector<Rational>(V.rows());
  
  perl::Object t("TropicalPolytope");
  t.set_description() << "Tropical polytope of " << p.name() <<endl;
 
  t.take("POINTS") << V;
  return t;
}

UserFunction4perl("# @category Other"
		  "# Takes an ordinary convex polytope and interprets it in tropical projective space."
 		  "# @param Polytope P"
		  "# @return TropicalPolytope",
 		  &poly2trop, "poly2trop(polytope::Polytope)");

} }

